import React, { PureComponent } from 'react';
import pathToRegexp from 'path-to-regexp';
import { Card, Row, Col, Form, Input, Button, Switch, Popover, Icon } from 'antd';
import { connect } from 'dva';
import { routerRedux } from 'dva/router';
import RBCityCascader from '../../components/RBCityCascader';
import FooterToolbar from '../../components/FooterToolbar';
import { RBSingleSelectTable } from '../../components/RBSelectTable';
import {
  renderStatus,
  renderCategory,
  provinceCodeToName,
  provinceNameToCode,
} from '../../utils/utils';
import styles from './CampusCreate.less';

const fieldLabels = {
  merchant: '所属商户',
  cityName: '所在城市',
  zoneName: '校区名称',
  contactName: '校区联系人',
  mobile: '联系电话',
  address: '收货地址',
  depositBank: '开户行',
  bankAccount: '银行账户',
};
const formItemLayout = {
  labelCol: {
    xs: { span: 24 },
    sm: { span: 2 },
  },
  wrapperCol: {
    xs: { span: 24 },
    sm: { span: 14 },
    md: { span: 10 },
  },
};
const submitFormLayout = {
  wrapperCol: {
    xs: { span: 24, offset: 0 },
    sm: { span: 12, offset: 6 },
  },
};

@Form.create()
@connect(({loading, merchant, campus}) => ({
  campus,
  merchant,
  loading: loading.models.merchant,
}))
export default class CampusCreatePage extends PureComponent {
  componentDidMount() {
    //加载商户列表
    this.props.dispatch({
      type: 'merchant/fetchMerchantList',
      payload: {},
    });
    //如果是编辑校区，加载校区详情
    const matchId = this.isEdit();
    if (matchId) {
      this.props.dispatch({
        type: 'campus/fetchCampusItem',
        payload: {id: matchId},
      });
    }
  }
  componentWillUnmount() {
    this.props.dispatch({
      type: 'campus/cleanItemState',
      payload: {},
    });
  }
  isEdit = () => {
    const { location } = this.props;
    const match = pathToRegexp('/campus/edit/:id').exec(location.pathname);
    if (match) {
      return match[1];
    }
    return false;
  }
  handleMerchantTableChange = (params) => {
    this.props.dispatch({
      type: 'merchant/fetchMerchantList',
      payload: params,
    });
  }
  handlePageSubmit = () => {
    this.props.form.validateFieldsAndScroll((error, values) => {
      if (!error) {
        const { cityName, ...restProps } = values;
        restProps.provinceCode = provinceNameToCode(cityName[0]);
        restProps.cityName = cityName[1];
        const matchId = this.isEdit();
        if (matchId) {
          restProps.id = matchId;
          this.props.dispatch({
            type: 'campus/updateCampusItem',
            payload: restProps,
            states: this.props.location.state,
          });
        } else {
          this.props.dispatch({
            type: 'campus/createCampusItem',
            payload: restProps,
            states: this.props.location.state,
          });
        }
      }
    });
  }
  handlePageBack = () => {
    this.props.dispatch(routerRedux.push({
      pathname: '/campus/list',
      state: this.props.location.state,
    }));
  }

  render() {
    const { form, merchant, loading, campus } = this.props;
    const { getFieldDecorator, getFieldsError } = form;
    const { list, totalSize, pageNo, pageSize } = merchant;
    const { currentItem } = campus;

    const pagination = { totalSize, pageSize, pageNo };
    const cardProps = {
      bordered: false,
      className: styles.cardItem,
    };

    const merchantColumns = [{
      title: '商户编号',
      dataIndex: 'code',
      key: 1,
    }, {
      title: '商户名称',
      dataIndex: 'name',
      key: 2,
    }, {
      title: '商户类型',
      key: 3,
      dataIndex: 'domain',
      render: (text) => renderCategory(text),
    }, {
      title: '该条状态',
      key: 4,
      dataIndex: 'status',
      render: (text) => renderStatus(text),
    }];

    const renderCityName = () => {
      const { provinceCode, cityName } = currentItem;
      if (!provinceCode && !cityName) {
        return;
      } else {
        return [
          provinceCodeToName(provinceCode),
          cityName,
        ];
      }
    }

    const errors = getFieldsError();
    const getErrorInfo = () => {
      const errorCount = Object.keys(errors).filter(key => errors[key]).length;
      if (!errors || errorCount === 0) {
        return null;
      }
      const scrollToField = (fieldKey) => {
        const labelNode = document.querySelector(`label[for="${fieldKey}"]`);
        if (labelNode) {
          labelNode.scrollIntoView(true);
        }
      };
      const errorList = Object.keys(errors).map((key) => {
        if (!errors[key]) {
          return null;
        }
        return (
          <li key={key} className={styles.errorListItem} onClick={() => scrollToField(key)}>
            <Icon type="cross-circle-o" className={styles.errorIcon} />
            <div className={styles.errorMessage}>{errors[key][0]}</div>
            <div className={styles.errorField}>{fieldLabels[key]}</div>
          </li>
        );
      });
      return (
        <span className={styles.errorIcon}>
          <Popover
            title="表单校验信息"
            content={errorList}
            overlayClassName={styles.errorPopover}
            trigger="click"
            getPopupContainer={trigger => trigger.parentNode}
          >
            <Icon type="exclamation-circle" />
          </Popover>
          {errorCount}
        </span>
      );
    };

    return (
      <div>
        <Card title={fieldLabels.merchant} {...cardProps} style={{marginBottom: 16}}>
          <Form>
            <Form.Item className={styles.firstFormItem}>
              {getFieldDecorator('merchantId', {
                rules: [{ required: true, message: '请选择商户!' }],
                initialValue: currentItem.merchantId,
              })(
                <RBSingleSelectTable
                  currentName={currentItem.merchantName}
                  columns={merchantColumns}
                  loading={loading}
                  dataSource={list}
                  pagination={pagination}
                  onFilterClick={this.handleMerchantTableChange}
                />
              )}
            </Form.Item>
          </Form>
        </Card>
        <Card title="基本信息" {...cardProps} style={{marginBottom: 70}}>
          <Form>
            <Row gutter={16}>
              <Col lg={8} md={12} sm={24}>
                <Form.Item label={fieldLabels.cityName}>
                  {getFieldDecorator('cityName', {
                    rules: [{ required: true, message: '请选择校区地址!' }],
                    initialValue: renderCityName(),
                  })(
                    <RBCityCascader />
                  )}
                </Form.Item>
              </Col>
              <Col lg={8} md={12} sm={24}>
                <Form.Item label={fieldLabels.zoneName}>
                  {getFieldDecorator('zoneName', {
                    rules: [{ required: true, message: '校区名不能为空!' }],
                    initialValue: currentItem.zoneName,
                  })(
                    <Input placeholder="请输入" />
                  )}
                </Form.Item>
              </Col>
              <Col lg={8} md={12} sm={24}>
                <Form.Item label={fieldLabels.contactName}>
                  {getFieldDecorator('contactName', {
                    rules: [{ required: true, message: '联系人不能为空!' }],
                    initialValue: currentItem.contactName,
                  })(
                    <Input placeholder="请输入" />
                  )}
                </Form.Item>
              </Col>
            </Row>
            <Row gutter={16}>
              <Col lg={8} md={12} sm={24}>
                <Form.Item label={fieldLabels.mobile}>
                  {getFieldDecorator('mobile', {
                    rules: [
                      {
                        required: true, message: '联系电话不能为空!',
                      }, {
                        pattern: /^[1][3,4,5,7,8][0-9]{9}$/g, message: '请输入11位有效手机号!',
                      }],
                    initialValue: currentItem.mobile,
                  })(
                    <Input placeholder="请输入" />
                  )}
                </Form.Item>
              </Col>
              <Col lg={8} md={12} sm={24}>
                <Form.Item label={fieldLabels.depositBank}>
                  {getFieldDecorator('depositBank', {
                    initialValue: currentItem.depositBank,
                  })(
                    <Input placeholder="请输入" />
                  )}
                </Form.Item>
              </Col>
              <Col lg={8} md={12} sm={24}>
                <Form.Item label={fieldLabels.bankAccount}>
                  {getFieldDecorator('bankAccount', {
                    initialValue: currentItem.bankAccount,
                  })(
                    <Input placeholder="请输入" />
                  )}
                </Form.Item>
              </Col>
            </Row>
            <Row gutter={16}>
              <Col lg={8} md={12} sm={24}>
                <Form.Item label={fieldLabels.address}>
                  {getFieldDecorator('address', {
                    initialValue: currentItem.address,
                  })(
                    <Input placeholder="请输入" />
                  )}
                </Form.Item>
              </Col>
            </Row>
          </Form>
        </Card>
        <FooterToolbar style={{width: '100%'}}>
          {getErrorInfo()}
          <Button onClick={this.handlePageBack} style={{marginRight: 10}}>
            取消
          </Button>
          <Button type="primary" onClick={this.handlePageSubmit}>
            提交
          </Button>
        </FooterToolbar>
      </div>
    );
  }
}
